#!/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

from abc import ABCMeta, abstractmethod, abstractproperty


class ISoup(object):
    """
    Class to work with UI objects.
    """

    __metaclass__ = ABCMeta

    @abstractproperty
    def mouse(self):
        """
        Instance of IMouse implementation.
        """

    @abstractproperty
    def keyboard(self):
        """
        Instance of IKeyboard implementation.
        """

    @abstractmethod
    def get_object_by_coordinates(self, x, y):
        """
        Gets object by coordinates.

        :param int x: x coordinate.
        :param int y: y coordinate.
        :rtype: uisoup.interfaces.i_element.IElement
        :return: object that was found by given coordinates.
        """

    @abstractmethod
    def is_window_exists(self, obj_handle):
        """
        Verifies is window exists.

        :param str | int obj_handle: window name (string) or window 
        handler (int) otherwise Desktop Window will be checked.
        :rtype: bool
        :return: True if window exists otherwise False.
        """

    @abstractmethod
    def get_window(self, obj_handle=None):
        """
        Gets window.

        :param str | int obj_handle: window name (string) or window 
        handler (int) otherwise Desktop Window will be checked.
        :rtype: uisoup.interfaces.i_element.IElement
        :return: window object.
        """

    @abstractmethod
    def get_visible_window_list(self):
        """
        Gets list of visible windows.

        :rtype: list[uisoup.interfaces.i_element.IElement]
        :return: list of visible windows.
        """

    @abstractmethod
    def get_visible_object_list(self, window_name):
        """
        Gets list of visible objects for specified window.

        :param str window_name: window name.
        :rtype: list[uisoup.interfaces.i_element.IElement]
        :return: list of visible windows.
        """



#!/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

import re
from inspect import ismethod
from types import FunctionType
from abc import ABCMeta, abstractmethod, abstractproperty
import xml.dom.minidom

from ..utils.common import CommonUtils

if CommonUtils.is_python_3():
    unicode = str


class IElement(object):
    """
    Class that describes UI object.
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def click(self, x_offset=None, y_offset=None):
        """
        Clicks by left mouse button on this object.

        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: y offset, if not defined half of element height 
        will be used.
        """

    @abstractmethod
    def right_click(self, x_offset=None, y_offset=None):
        """
        Clicks by right mouse button on this object.

        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: y offset, if not defined half of element height 
        will be used.
        """

    @abstractmethod
    def double_click(self, x_offset=None, y_offset=None):
        """
        Double clicks by left mouse button on this object.

        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: y offset, if not defined half of element height 
        will be used
        """

    @abstractmethod
    def drag_to(self, x, y, x_offset=None, y_offset=None, smooth=True):
        """
        Drags this object to coordinates.

        :param int x: x coordinate.
        :param int y: y coordinate.
        :param int x_offset: if not defined half of element width will be used.
        :param int y_offset: if not defined half of element height 
        will be used.
        :param bool smooth: indicates is it needed to simulate smooth movement.
        """

    @abstractproperty
    def proc_id(self):
        """
        Indicates process id.
        """

    @abstractproperty
    def is_top_level_window(self):
        """
        Indicates is top level window or not.
        """

    @abstractproperty
    def is_selected(self):
        """
        Indicates selected state.
        """

    @abstractproperty
    def is_checked(self):
        """
        Indicates checked state.
        """

    @abstractproperty
    def is_visible(self):
        """
        Indicates visible state.
        """

    @abstractproperty
    def is_enabled(self):
        """
        Indicates enabled state.
        """

    @abstractproperty
    def acc_parent_count(self):
        """
        Property for parent child count.
        """

    @abstractproperty
    def acc_child_count(self):
        """
        Property for element child count.
        """

    @abstractproperty
    def acc_name(self):
        """
        Property for element name.
        Also need to specify setter for this property
        """

    @abstractmethod
    def set_focus(self):
        """
        Sets focus to element.
        """

    @abstractproperty
    def acc_c_name(self):
        """
        Property for combined name (role name + name).
        """

    @abstractproperty
    def acc_location(self):
        """
        Property for element location.
        """

    @abstractproperty
    def acc_value(self):
        """
        Property for element value.
        Also need to specify setter for this property.
        """

    @abstractmethod
    def set_value(self, value):
        """
        Sets element value.

        :param str value: element value.
        """

    @abstractproperty
    def acc_description(self):
        """
        Property for element description.
        """

    @abstractproperty
    def acc_parent(self):
        """
        Property for element parent.
        """

    @abstractproperty
    def acc_selection(self):
        """
        Property for element selection.
        """

    @abstractproperty
    def acc_focused_element(self):
        """
        Property for element in focus.
        """

    @abstractproperty
    def acc_role_name(self):
        """
        Property for element role name.
        """

    @abstractmethod
    def __iter__(self):
        """Iterate all child Element"""

    @abstractmethod
    def find(self, only_visible=True, **kwargs):
        """
        Finds first child element.

        :param bool only_visible: flag that indicates will we search only
        through visible elements.
        :param str role: string or lambda e.g. lambda x: x == 13
        :param str name: string or lambda.
        :param str c_name: string or lambda.
        :param str location: string or lambda.
        :param str value: string or lambda.
        :param str description: string or lambda.
        :param str selection: string or lambda.
        :param str role_name: string or lambda.
        :param str parent_count: string or lambda.
        :param str child_count: string or lambda.
        :rtype: IElement
        :return: Element that was found otherwise exception will be raised.
        """

    @abstractmethod
    def findall(self, only_visible=True, **kwargs):
        """
        Find all child element.

        :param bool only_visible: flag that indicates will we search only
        through visible elements.
        :param str role: string or lambda e.g. lambda x: x == 13
        :param str name: string or lambda.
        :param str c_name: string or lambda.
        :param str location: string or lambda.
        :param str value: string or lambda.
        :param str description: string or lambda.
        :param str selection: string or lambda.
        :param str role_name: string or lambda.
        :param str parent_count: string or lambda.
        :param str child_count: string or lambda.
        :rtype: list[IElement]
        :return: List of all elements that was found otherwise None.
        """

    @abstractmethod
    def is_object_exists(self, **kwargs):
        """
        Verifies is object exists.

        :param bool only_visible: flag that indicates will we search only
        through visible elements.
        :param str role: string or lambda e.g. lambda x: x == 13
        :param str name: string or lambda.
        :param str c_name: string or lambda.
        :param str location: string or lambda.
        :param str value: string or lambda.
        :param str description: string or lambda.
        :param str selection: string or lambda.
        :param str role_name: string or lambda.
        :param str parent_count: string or lambda.
        :param str child_count: string or lambda.
        :rtype: bool
        :return: True if object exists otherwise False.
        """

    def toxml(self):
        """
        Convert Element Tree to XML.
        """
        obj_document = xml.dom.minidom.Document()
        lst_queue = [(self, obj_document)]

        while lst_queue:
            obj_element, obj_tree = lst_queue.pop(0)
            role_name = obj_element.acc_role_name
            obj_name = obj_element.acc_name
            str_name = unicode(obj_name) if obj_name else ''
            str_location = ','.join(str(x) for x in obj_element.acc_location)
            obj_sub_tree = xml.dom.minidom.Element(role_name)
            obj_sub_tree.ownerDocument = obj_document

            try:
                obj_sub_tree.attributes['Name'] = str_name
            except:
                obj_sub_tree.attributes['Name'] = \
                    str_name.encode('unicode-escape')

            obj_sub_tree.attributes['Location'] = str_location
            obj_tree.appendChild(obj_sub_tree)

            if obj_element.acc_child_count:
                for obj_element_child in obj_element:
                    lst_queue.append((obj_element_child, obj_sub_tree))

        return obj_document.toprettyxml()

    def __str__(self):
        result = '[Role: %s | Name: %r | Child count: %d]' % \
                 (self.acc_role_name,
                  self.acc_name,
                  self.acc_child_count)

        return result

    def _match(self, only_visible, **kwargs):
        """
        Match method.

        :param bool only_visible: flag that indicates will we search only
        through visible elements.
        :param str role: string or lambda e.g. lambda x: x == 13
        :param str name: string or lambda.
        :param str c_name: string or lambda.
        :param str location: string or lambda.
        :param str value: string or lambda.
        :param str description: string or lambda.
        :param str selection: string or lambda.
        :param str role_name: string or lambda.
        :param str parent_count: string or lambda.
        :param str child_count: string or lambda.
        :rtype: bool
        :return: True if element was matched otherwise False.
        """
        try:
            if only_visible and not self.is_visible:
                return False

            for str_property, expected_result in kwargs.items():
                attr = getattr(self, 'acc_' + str_property)
                if ismethod(attr):
                    attr = attr()

                if type(expected_result) is FunctionType:
                    if not expected_result(attr):
                        return False
                else:
                    regex = CommonUtils.convert_wildcard_to_regex(
                        expected_result)
                    if not re.match(regex, attr):
                        return False
        except:
            return False
        else:
            return True



#!/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

import ctypes
from time import sleep

from ..interfaces.i_keyboard import Key, IKeyboard

send_input = ctypes.windll.user32.SendInput
pointer_unsigned_long = ctypes.POINTER(ctypes.c_ulong)


class KeyboardInput(ctypes.Structure):
    """
    Keyboard input C struct definition.
    """

    _fields_ = [("wVk", ctypes.c_ushort),
                ("wScan", ctypes.c_ushort),
                ("dwFlags", ctypes.c_ulong),
                ("time", ctypes.c_ulong),
                ("dwExtraInfo", pointer_unsigned_long)]


class HardwareInput(ctypes.Structure):
    """
    Hardware input C struct definition.
    """

    _fields_ = [("uMsg", ctypes.c_ulong),
                ("wParamL", ctypes.c_short),
                ("wParamH", ctypes.c_ushort)]


class MouseInput(ctypes.Structure):
    """
    Hardware input C struct definition.
    """

    _fields_ = [("dx", ctypes.c_long),
                ("dy", ctypes.c_long),
                ("mouseData", ctypes.c_ulong),
                ("dwFlags", ctypes.c_ulong),
                ("time", ctypes.c_ulong),
                ("dwExtraInfo", pointer_unsigned_long)]


class EventStorage(ctypes.Union):
    """
    Event storage C struct definition.
    """

    _fields_ = [("ki", KeyboardInput),
                ("mi", MouseInput),
                ("hi", HardwareInput)]


class Input(ctypes.Structure):
    """
    Input C struct definition.
    """

    _fields_ = [("type", ctypes.c_ulong),
                ("ii", EventStorage)]


class WinKeyboard(IKeyboard):

    class _KeyCodes(object):
        """
        Holder for Windows keyboard codes stored as Keys.
        """

        BACKSPACE = Key(0x08)  # BACKSPACE key
        TAB = Key(0x09)  # TAB key
        CLEAR = Key(0x0C)  # CLEAR key
        RETURN = Key(0x0D)  # ENTER key
        SHIFT = Key(0x10)  # SHIFT key
        CONTROL = Key(0x11)  # CTRL key
        ALT = Key(0x12)  # ALT key
        PAUSE = Key(0x13)  # PAUSE key
        CAPS_LOCK = Key(0x14)  # CAPS LOCK key
        ESCAPE = Key(0x1B)  # ESC key
        SPACE = Key(0x20)  # SPACEBAR
        PAGE_UP = Key(0x21)  # PAGE UP key
        PAGE_DOWN = Key(0x22)  # PAGE DOWN key
        END = Key(0x23)  # END key
        HOME = Key(0x24)  # HOME key
        LEFT = Key(0x25)  # LEFT ARROW key
        UP = Key(0x26)  # UP ARROW key
        RIGHT = Key(0x27)  # RIGHT ARROW key
        DOWN = Key(0x28)  # DOWN ARROW key
        PRINT_SCREEN = Key(0x2C)  # PRINT SCREEN key
        INSERT = Key(0x2D)  # INS key
        DELETE = Key(0x2E)  # DEL key
        VK_HELP = Key(0x2F)  # HELP key
        KEY_0 = Key(0x30)  # 0 key
        KEY_1 = Key(0x31)  # 1 key
        KEY_2 = Key(0x32)  # 2 key
        KEY_3 = Key(0x33)  # 3 key
        KEY_4 = Key(0x34)  # 4 key
        KEY_5 = Key(0x35)  # 5 key
        KEY_6 = Key(0x36)  # 6 key
        KEY_7 = Key(0x37)  # 7 key
        KEY_8 = Key(0x38)  # 8 key
        KEY_9 = Key(0x39)  # 9 key
        KEY_A = Key(0x41)  # A key
        KEY_B = Key(0x42)  # B key
        KEY_C = Key(0x43)  # C key
        KEY_D = Key(0x44)  # D key
        KEY_E = Key(0x45)  # E key
        KEY_F = Key(0x46)  # F key
        KEY_G = Key(0x47)  # G key
        KEY_H = Key(0x48)  # H key
        KEY_I = Key(0x49)  # I key
        KEY_J = Key(0x4A)  # J key
        KEY_K = Key(0x4B)  # K key
        KEY_L = Key(0x4C)  # L key
        KEY_M = Key(0x4D)  # M key
        KEY_N = Key(0x4E)  # N key
        KEY_O = Key(0x4F)  # O key
        KEY_P = Key(0x50)  # P key
        KEY_Q = Key(0x51)  # Q key
        KEY_R = Key(0x52)  # R key
        KEY_S = Key(0x53)  # S key
        KEY_T = Key(0x54)  # T key
        KEY_U = Key(0x55)  # U key
        KEY_V = Key(0x56)  # V key
        KEY_W = Key(0x57)  # W key
        KEY_X = Key(0x58)  # X key
        KEY_Y = Key(0x59)  # Y key
        KEY_Z = Key(0x5A)  # Z key
        LEFT_WIN = Key(0x5B)  # Left Windows key (Natural keyboard)
        RIGHT_WIN = Key(0x5C)  # Right Windows key (Natural keyboard)
        SLEEP = Key(0x5F)  # Computer Sleep key
        NUMPAD0 = Key(0x60)  # Numeric keypad 0 key
        NUMPAD1 = Key(0x61)  # Numeric keypad 1 key
        NUMPAD2 = Key(0x62)  # Numeric keypad 2 key
        NUMPAD3 = Key(0x63)  # Numeric keypad 3 key
        NUMPAD4 = Key(0x64)  # Numeric keypad 4 key
        NUMPAD5 = Key(0x65)  # Numeric keypad 5 key
        NUMPAD6 = Key(0x66)  # Numeric keypad 6 key
        NUMPAD7 = Key(0x67)  # Numeric keypad 7 key
        NUMPAD8 = Key(0x68)  # Numeric keypad 8 key
        NUMPAD9 = Key(0x69)  # Numeric keypad 9 key
        MULTIPLY = Key(0x6A)  # Multiply key
        ADD = Key(0x6B)  # Add key
        SEPARATOR = Key(0x6C)  # Separator key
        SUBTRACT = Key(0x6D)  # Subtract key
        DECIMAL = Key(0x6E)  # Decimal key
        DIVIDE = Key(0x6F)  # Divide key
        F1 = Key(0x70)  # F1 key
        F2 = Key(0x71)  # F2 key
        F3 = Key(0x72)  # F3 key
        F4 = Key(0x73)  # F4 key
        F5 = Key(0x74)  # F5 key
        F6 = Key(0x75)  # F6 key
        F7 = Key(0x76)  # F7 key
        F8 = Key(0x77)  # F8 key
        F9 = Key(0x78)  # F9 key
        F10 = Key(0x79)  # F10 key
        F11 = Key(0x7A)  # F11 key
        F12 = Key(0x7B)  # F12 key
        NUM_LOCK = Key(0x90)  # NUM LOCK key
        SCROLL_LOCK = Key(0x91)  # SCROLL LOCK
        LEFT_SHIFT = Key(0xA0)  # Left SHIFT key
        RIGHT_SHIFT = Key(0xA1)  # Right SHIFT key
        LEFT_CONTROL = Key(0xA2)  # Left CONTROL key
        RIGHT_CONTROL = Key(0xA3)  # Right CONTROL key
        OEM_1 = Key(0xBA)  # For the US standard keyboard, the ';:' key
        OEM_PLUS = Key(0xBB)  # For any country/region, the '+' key
        OEM_COMMA = Key(0xBC)  # For any country/region, the ',' key
        OEM_MINUS = Key(0xBD)  # For any country/region, the '-' key
        OEM_PERIOD = Key(0xBE)  # For any country/region, the '.' key
        OEM_2 = Key(0xBF)  # For the US standard keyboard, the '/?' key
        OEM_3 = Key(0xC0)  # For the US standard keyboard, the '`~' key
        OEM_4 = Key(0xDB)  # For the US standard keyboard, the '[{' key
        OEM_5 = Key(0xDC)  # For the US standard keyboard, the '\|' key
        OEM_6 = Key(0xDD)  # For the US standard keyboard, the ']}' key
        OEM_7 = Key(0xDE)  # For the US standard keyboard, the ''/"' key

    codes = _KeyCodes

    def press_key(self, hex_key_code):
        """
        Presses (and releases) key specified by a hex code.

        :param int hex_key_code: hexadecimal code for a key to be pressed.
        """
        self.press_key_and_hold(hex_key_code)
        self.release_key(hex_key_code)

    def press_key_and_hold(self, hex_key_code):
        """
        Presses (and holds) key specified by a hex code.

        :param int hex_key_code: hexadecimal code for a key to be pressed.
        """
        extra = ctypes.c_ulong(0)
        ii_ = EventStorage()
        ii_.ki = KeyboardInput(hex_key_code, 0x48, 0, 0, ctypes.pointer(extra))
        x = Input(ctypes.c_ulong(1), ii_)
        send_input(1, ctypes.pointer(x), ctypes.sizeof(x))

    def release_key(self, hex_key_code):
        """
        Releases key specified by a hex code.

        :param int hex_key_code: hexadecimal code for a key to be pressed.
        """
        extra = ctypes.c_ulong(0)
        ii_ = EventStorage()
        ii_.ki = KeyboardInput(
            hex_key_code, 0x48, 0x0002, 0, ctypes.pointer(extra))
        x = Input(ctypes.c_ulong(1), ii_)
        send_input(1, ctypes.pointer(x), ctypes.sizeof(x))

    def send(self, *args, **kwargs):
        """
        Send key events as specified by Keys.

        If Key contains children Keys they will be recursively
        processed with current Key code pressed as a modifier key.

        :param args: keys to send.
        """
        delay = kwargs.get('delay', 0)

        for key in args:
            if key.children:
                self.press_key_and_hold(key.code)
                self.send(*key.children)
                self.release_key(key.code)
            else:
                self.press_key(key.code)
            self._wait_for_key_combo_to_be_processed()
            sleep(delay)

    def _wait_for_key_combo_to_be_processed(self):
        # For key combinations timeout is needed to be processed.
        # This method is expressive shortcut to be used where needed.
        sleep(.05)



#!/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

import ctypes
import ctypes.wintypes
import comtypes
import comtypes.automation
import comtypes.client

from .mouse import WinMouse
from ..interfaces.i_element import IElement
from ..utils.win_utils import WinUtils
from .. import TooSaltyUISoupException

if WinUtils.is_python_3():
    xrange = range

CO_E_OBJNOTCONNECTED = -2147220995


class WinElement(IElement):
    """
    http://msdn.microsoft.com/en-us/library/dd318466(v=VS.85).aspx
    """

    _acc_role_name_map = {
        1: u'ttl',  # TitleBar
        2: u'mnu',  # MenuBar
        3: u'scbr',  # ScrollBar
        4: u'grip',  # Grip
        5: u'snd',  # Sound
        6: u'cur',  # Cursor
        7: u'caret',  # Caret
        8: u'alrt',  # Alert
        9: u'frm',  # Window
        10: u'clnt',  # Client
        11: u'mnu',  # PopupMenu
        12: u'mnu',  # MenuItem
        13: u'ttip',  # Tooltip
        14: u'app',  # Application
        15: u'doc',  # Document
        16: u'pane',  # Pane
        17: u'chrt',  # Chart
        18: u'dlg',  # Dialog
        19: u'border',  # Border
        20: u'grp',  # Grouping
        21: u'sep',  # Separator
        22: u'tbar',  # ToolBar
        23: u'sbar',  # StatusBar
        24: u'tbl',  # Table
        25: u'chdr',  # ColumnHeader
        26: u'rhdr',  # RowHeader
        27: u'col',  # Column
        28: u'tblc',  # Row
        29: u'tblc',  # Cell
        30: u'lnk',  # Link
        31: u'hbal',  # HelpBalloon
        32: u'chr',  # Character
        33: u'lst',  # List
        34: u'lst',  # ListItem
        35: u'otl',  # Outline
        36: u'otl',  # OutlineItem
        37: u'ptab',  # PageTab
        38: u'ppage',  # PropertyPage
        39: u'val',  # Indicator
        40: u'grph',  # Graphic
        41: u'lbl',  # Text
        42: u'txt',  # EditableText
        43: u'btn',  # PushButton
        44: u'chk',  # CheckBox
        45: u'rbtn',  # RadioButton
        46: u'cbox',  # ComboBox
        47: u'ddwn',  # DropDown
        48: u'pbar',  # ProgressBar
        49: u'dial',  # Dial
        50: u'hkfield',  # HotKeyField
        51: u'sldr',  # Slider
        52: u'sbox',  # SpinBox
        53: u'dgrm',  # Diagram
        54: u'anim',  # Animation
        55: u'eqtn',  # Equation
        56: u'btn',  # DropDownButton
        57: u'mnu',  # MenuButton
        58: u'gbtn',  # GridDropDownButton
        59: u'wspace',  # WhiteSpace
        60: u'ptablst',  # PageTabList
        61: u'clock',  # Clock
        62: u'sbtn',  # SplitButton
        63: u'ip',  # IPAddress
        64: u'obtn'  # OutlineButton
    }

    _mouse = WinMouse()

    class _StateFlag(object):
        SYSTEM_NORMAL = 0
        SYSTEM_UNAVAILABLE = 0x1
        SYSTEM_SELECTED = 0x2
        SYSTEM_FOCUSED = 0x4
        SYSTEM_PRESSED = 0x8
        SYSTEM_CHECKED = 0x10
        SYSTEM_MIXED = 0x20
        SYSTEM_READONLY = 0x40
        SYSTEM_HOTTRACKED = 0x80
        SYSTEM_DEFAULT = 0x100
        SYSTEM_EXPANDED = 0x200
        SYSTEM_COLLAPSED = 0x400
        SYSTEM_BUSY = 0x800
        SYSTEM_FLOATING = 0x1000
        SYSTEM_MARQUEED = 0x2000
        SYSTEM_ANIMATED = 0x4000
        SYSTEM_INVISIBLE = 0x8000
        SYSTEM_OFFSCREEN = 0x10000
        SYSTEM_SIZEABLE = 0x20000
        SYSTEM_MOVEABLE = 0x40000
        SYSTEM_SELFVOICING = 0x80000
        SYSTEM_FOCUSABLE = 0x100000
        SYSTEM_SELECTABLE = 0x200000
        SYSTEM_LINKED = 0x400000
        SYSTEM_TRAVERSED = 0x800000
        SYSTEM_MULTISELECTABLE = 0x1000000
        SYSTEM_EXTSELECTABLE = 0x2000000
        SYSTEM_ALERT_LOW = 0x4000000
        SYSTEM_ALERT_MEDIUM = 0x8000000
        SYSTEM_ALERT_HIGH = 0x10000000
        SYSTEM_PROTECTED = 0x20000000
        SYSTEM_HASPOPUP = 0x40000000
        SYSTEM_VALID = 0x7fffffff

    class _SelectionFlag(object):
        NONE = 0
        TAKEFOCUS = 0x1
        TAKESELECTION = 0x2
        EXTENDSELECTION = 0x4
        ADDSELECTION = 0x8
        REMOVESELECTION = 0x10
        VALID = 0x20

    class _EnumWindowsCallback(object):

        same_proc_handles = set()

        @classmethod
        def callback(cls, handle, proc_id):

            curr_proc_id = ctypes.c_long()

            ctypes.windll.user32.GetWindowThreadProcessId(
                handle, ctypes.byref(curr_proc_id))

            if curr_proc_id.value == proc_id:
                cls.same_proc_handles.add(handle)

            return True

    def __init__(self, obj_handle, i_object_id):
        """
        Constructor.

        :param obj_handle: instance of i_accessible or window handle.
        :param int i_object_id: object id.
        """
        if isinstance(obj_handle, comtypes.gen.Accessibility.IAccessible):
            i_accessible = obj_handle
        else:
            i_accessible = ctypes.POINTER(
                comtypes.gen.Accessibility.IAccessible)()
            ctypes.oledll.oleacc.AccessibleObjectFromWindow(
                obj_handle,
                0,
                ctypes.byref(comtypes.gen.Accessibility.IAccessible._iid_),
                ctypes.byref(i_accessible))

        self._i_accessible = i_accessible
        self._i_object_id = i_object_id
        self._cached_children = set()

    def _check_state(self, state):
        """
        Checks state.

        :param int state: state flag.

        :rtype: bool
        :return: bool flag indicator.
        """
        return bool(self._acc_state & state)

    def _find_windows_by_same_proc(self):
        """
        Find window by same process id.

        :rtype: list
        :return: list of windows.
        """
        enum_windows_proc = \
            ctypes.WINFUNCTYPE(ctypes.c_bool, ctypes.c_long,
                               ctypes.c_long)
        self._EnumWindowsCallback.same_proc_handles = set()
        ctypes.windll.user32.EnumWindows(
            enum_windows_proc(
                self._EnumWindowsCallback.callback), self.proc_id)

        if self._hwnd in self._EnumWindowsCallback.same_proc_handles:
            self._EnumWindowsCallback.same_proc_handles.remove(self._hwnd)

        result = [WinElement(hwnd, 0) for hwnd in
                  self._EnumWindowsCallback.same_proc_handles]

        return result

    @property
    def _hwnd(self):
        """
        Property for window handler.
        """
        hwnd = ctypes.c_int()
        ctypes.oledll.oleacc.WindowFromAccessibleObject(self._i_accessible,
                                                        ctypes.byref(hwnd))

        return hwnd.value

    @property
    def _role(self):
        """
        Property for element role.
        """
        obj_child_id = comtypes.automation.VARIANT()
        obj_child_id.vt = comtypes.automation.VT_I4
        obj_child_id.value = self._i_object_id
        obj_role = comtypes.automation.VARIANT()
        obj_role.vt = comtypes.automation.VT_BSTR

        self._i_accessible._IAccessible__com__get_accRole(obj_child_id,
                                                          obj_role)

        return obj_role.value

    def _select(self, i_selection):
        if self._i_object_id:
            return self._i_accessible.accSelect(i_selection, self._i_object_id)
        else:
            return self._i_accessible.accSelect(i_selection)

    def click(self, x_offset=0, y_offset=0):
        x, y, w, h = self.acc_location
        x += x_offset if x_offset is not None else w / 2
        y += y_offset if y_offset is not None else h / 2

        self._mouse.click(x, y)

    def right_click(self, x_offset=0, y_offset=0):
        x, y, w, h = self.acc_location
        x += x_offset if x_offset is not None else w / 2
        y += y_offset if y_offset is not None else h / 2

        self._mouse.click(x, y, self._mouse.RIGHT_BUTTON)

    def double_click(self, x_offset=0, y_offset=0):
        x, y, w, h = self.acc_location
        x += x_offset if x_offset is not None else w / 2
        y += y_offset if y_offset is not None else h / 2

        self._mouse.double_click(x, y)

    def drag_to(self, x, y, x_offset=None, y_offset=None, smooth=True):
        el_x, el_y, el_w, el_h = self.acc_location
        el_x += x_offset if x_offset is not None else el_w / 2
        el_y += y_offset if y_offset is not None else el_h / 2

        self._mouse.drag(el_x, el_y, x, y, smooth)

    @property
    def proc_id(self):
        hwnd = ctypes.c_long(self._hwnd)
        proc_id = ctypes.c_ulong()
        ctypes.windll.user32.GetWindowThreadProcessId(hwnd,
                                                      ctypes.byref(proc_id))

        return proc_id.value

    @property
    def is_top_level_window(self):
        # Top level window have 2 parents, clnt and frm for Desktop.
        return self.acc_parent_count == 2

    @property
    def is_selected(self):
        return self._check_state(self._StateFlag.SYSTEM_SELECTED)

    @property
    def is_checked(self):
        return self._check_state(self._StateFlag.SYSTEM_CHECKED)

    @property
    def is_visible(self):
        return not self._check_state(self._StateFlag.SYSTEM_INVISIBLE)

    @property
    def is_enabled(self):
        return not self._check_state(self._StateFlag.SYSTEM_UNAVAILABLE)

    @property
    def acc_parent_count(self):
        parent_count = 0
        parent = self.acc_parent
        while parent:
            parent_count += 1
            parent = parent.acc_parent

        return parent_count

    @property
    def acc_child_count(self):
        if self._i_object_id == 0:
            return self._get_child_count_safely(self._i_accessible)
        else:
            return 0

    @property
    def acc_name(self):
        obj_child_id = comtypes.automation.VARIANT()
        obj_child_id.vt = comtypes.automation.VT_I4
        obj_child_id.value = self._i_object_id

        obj_name = comtypes.automation.BSTR()

        self._i_accessible._IAccessible__com__get_accName(
            obj_child_id, ctypes.byref(obj_name))
        result = obj_name.value or ''

        return WinUtils.replace_inappropriate_symbols(result)

    def set_focus(self):
        self._select(self._SelectionFlag.TAKEFOCUS)

    @property
    def acc_c_name(self):
        return self.acc_role_name + self.acc_name if self.acc_name else ''

    @property
    def acc_location(self):
        obj_child_id = comtypes.automation.VARIANT()
        obj_child_id.vt = comtypes.automation.VT_I4
        obj_child_id.value = self._i_object_id

        obj_l, obj_t, obj_w, obj_h = ctypes.c_long(), ctypes.c_long(), \
            ctypes.c_long(), ctypes.c_long()

        self._i_accessible._IAccessible__com_accLocation(ctypes.byref(obj_l),
                                                         ctypes.byref(obj_t),
                                                         ctypes.byref(obj_w),
                                                         ctypes.byref(obj_h),
                                                         obj_child_id)

        return obj_l.value, obj_t.value, obj_w.value, obj_h.value

    @property
    def acc_value(self):
        obj_child_id = comtypes.automation.VARIANT()
        obj_child_id.vt = comtypes.automation.VT_I4
        obj_child_id.value = self._i_object_id
        obj_bstr_value = comtypes.automation.BSTR()
        self._i_accessible._IAccessible__com__get_accValue(
            obj_child_id, ctypes.byref(obj_bstr_value))

        return obj_bstr_value.value

    def set_value(self, value):
        obj_child_id = comtypes.automation.VARIANT()
        obj_child_id.vt = comtypes.automation.VT_I4
        obj_child_id.value = self._i_object_id

        self._i_accessible._IAccessible__com__set_accValue(obj_child_id, value)

    @property
    def acc_description(self):
        obj_child_id = comtypes.automation.VARIANT()
        obj_child_id.vt = comtypes.automation.VT_I4
        obj_child_id.value = self._i_object_id
        obj_description = comtypes.automation.BSTR()
        self._i_accessible._IAccessible__com__get_accDescription(
            obj_child_id, ctypes.byref(obj_description))

        return obj_description.value

    @property
    def acc_parent(self):
        result = None
        if self._i_accessible.accParent:
            result = \
                WinElement(self._i_accessible.accParent, self._i_object_id)

        return result

    @property
    def acc_selection(self):
        obj_children = comtypes.automation.VARIANT()
        self._i_accessible._IAccessible__com__get_accSelection(
            ctypes.byref(obj_children))

        return obj_children.value

    @property
    def _acc_state(self):
        obj_child_id = comtypes.automation.VARIANT()
        obj_child_id.vt = comtypes.automation.VT_I4
        obj_child_id.value = self._i_object_id
        obj_state = comtypes.automation.VARIANT()
        self._i_accessible._IAccessible__com__get_accState(
            obj_child_id, ctypes.byref(obj_state))

        return obj_state.value

    @property
    def acc_focused_element(self):
        result = None
        if self._i_accessible.accFocus:
            result = WinElement(self._i_accessible.accFocus, self._i_object_id)

        return result

    @property
    def acc_role_name(self):
        return self._acc_role_name_map.get(self._role, 'unknown')

    def __iter__(self):
        if self._i_object_id > 0:
            raise StopIteration()

        obj_acc_child_array = (comtypes.automation.VARIANT *
                               self._i_accessible.accChildCount)()
        obj_acc_child_count = ctypes.c_long()

        ctypes.oledll.oleacc.AccessibleChildren(
            self._i_accessible,
            0,
            self._i_accessible.accChildCount,
            obj_acc_child_array,
            ctypes.byref(obj_acc_child_count))

        for i in xrange(obj_acc_child_count.value):
            obj_acc_child = obj_acc_child_array[i]
            if obj_acc_child.vt == comtypes.automation.VT_DISPATCH:
                yield WinElement(obj_acc_child.value.QueryInterface(
                    comtypes.gen.Accessibility.IAccessible), 0)
            else:
                yield WinElement(self._i_accessible, obj_acc_child.value)

    def __findcacheiter(self, only_visible, **kwargs):
        """
        Find child element in the cache.

        :param bool only_visible: flag that indicates will we search only
        :rtype: WinElement
        :return: yield found element.
        """
        for obj_element in self._cached_children:
            if obj_element._match(only_visible, **kwargs):
                yield obj_element

    def _finditer(self, only_visible, **kwargs):
        """
        Find child element.

        :param bool only_visible: flag that indicates will we search only
        :rtype: WinElement
        :return: yield found element.
        """
        lst_queue = list(self)

        if self.is_top_level_window:
            lst_queue.extend(self._find_windows_by_same_proc())

        while lst_queue:
            obj_element = lst_queue.pop(0)
            self._cached_children.add(obj_element)

            if obj_element._match(only_visible, **kwargs):
                yield obj_element

            if obj_element.acc_child_count:
                childs = [el for el in list(obj_element) if
                          el._i_accessible != obj_element._i_accessible]
                lst_queue[:0] = childs

    def find(self, only_visible=True, **kwargs):
        try:
            iter_ = next(self.__findcacheiter(only_visible, **kwargs)) if \
                WinUtils.is_python_3() else self.__findcacheiter(
                only_visible, **kwargs).next()
            return iter_
        except StopIteration:
            try:
                iter_ = next(self._finditer(only_visible, **kwargs)) if \
                    WinUtils.is_python_3() else self.__findcacheiter(
                    only_visible, **kwargs).next()
                return iter_
            except StopIteration:
                items_ = kwargs.items() if WinUtils.is_python_3() else \
                    kwargs.iteritems()
                attrs = ['%s=%s' % (k, v) for k, v in items_]
                raise TooSaltyUISoupException(
                    'Can\'t find object with attributes "%s".' %
                    '; '.join(attrs))

    def findall(self, only_visible=True, **kwargs):
        result = self._finditer(only_visible, **kwargs)
        if result:
            result = list(result)

        return result

    def is_object_exists(self, **kwargs):
        try:
            self.find(**kwargs)
            return True
        except TooSaltyUISoupException:
            return False

    def _get_child_count_safely(self, i_accessible):
        """
        Safely gets child count.

        :param i_accessible: instance of i_accessible.
        :rtype: int
        :return: object child count
        """
        try:
            return i_accessible.accChildCount
        except Exception as ex:
            if isinstance(ex, comtypes.COMError) and getattr(ex, 'hresult') \
                    in (CO_E_OBJNOTCONNECTED,):
                return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'


from os import system
from time import sleep
from inspect import ismethod
from platform import system as platform_system

from . import uisoup


class UIInspector(object):

    @classmethod
    def get_current_element_info(cls, obj_element):
        """
        Gets current element info.

        :param obj_element: object element.
        :rtype: str
        :return: element attributes.
        """
        dict_info = {}
        lst_attribute_name_list = ['acc_role_name',
                                   'acc_name',
                                   'acc_value',
                                   'acc_location',
                                   'acc_description',
                                   'acc_child_count']

        for attr in lst_attribute_name_list:
            try:
                value = getattr(obj_element, attr)
                if ismethod(value):
                    dict_info[attr] = value()
                else:
                    dict_info[attr] = value
            except:
                dict_info[attr] = None

        return '\n'.join('%s:\t%r' % (attr, dict_info[attr]) for
                         attr in lst_attribute_name_list)


def main():
    """
    Starts UI Inspector.
    """

    try:
        x_old, y_old = None, None
        while True:
            x, y = uisoup.mouse.get_position()
            if (x, y) != (x_old, y_old):
                x_old, y_old = x, y
                obj_element = uisoup.get_object_by_coordinates(x, y)
                clear_command = \
                    'cls' if platform_system() == 'Windows' else 'clear'
                printable_data = \
                    UIInspector.get_current_element_info(obj_element)
                system(clear_command)
                print(printable_data)
            sleep(0.5)
    except KeyboardInterrupt:
        system('cls')



# !/usr/bin/env python

#    Copyright (c) 2014-2017 Max Beloborodko.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__author__ = 'f1ashhimself@gmail.com'

import re
import sys

from .. import TooSaltyUISoupException


class CommonUtils(object):

    @classmethod
    def convert_wildcard_to_regex(cls, wildcard):
        """
        Converts wildcard to regex.

        :param str wildcard: wildcard.
        :rtype: str
        :return: regex pattern.
        """
        regex = re.escape(wildcard)
        regex = regex.replace(r'\?', r'[\s\S]{1}')
        regex = regex.replace(r'\*', r'[\s\S]*')

        return '^%s$' % regex

    @classmethod
    def replace_inappropriate_symbols(cls, text):
        """
        Replaces inappropriate symbols e.g. \xa0 (non-breaking space) to
        normal space.

        :param str text: text in which symbols should be replaced.
        :rtype: str
        :return: processed text.
        """
        replace_pairs = [(u'\xa0', ' '),
                         (u'\u2014', '-')]

        for from_, to_ in replace_pairs:
            text = text.replace(from_, to_)

        return text

    @classmethod
    def verify_xy_coordinates(cls, x, y):
        """
        Verifies that x and y is instance of int otherwise raises exception.

        :param x: x variable.
        :param y: y variable.
        """
        if not isinstance(x, (int, float)) or not isinstance(y, (int, float)):
            raise TooSaltyUISoupException(
                'x and y arguments should hold int coordinates.')

    @classmethod
    def verify_mouse_button_name(cls, button_name, supported_names):
        """
        Verifies that button name is supported otherwise raises exception.

        :param str button_name: button name.
        :param list[str] supported_names: supported button names.
        """
        if button_name not in supported_names:
            raise TooSaltyUISoupException(
                'Button name should be one of supported %s.' %
                repr(supported_names))

    @classmethod
    def is_python_3(cls):
        """
        Indicates is currently python 3 used or not.
        
        :rtype: bool
        :return: boolean indicator. 
        """
        return sys.version_info.major == 3



#!/usr/bin/env python

from os import path

from setuptools import setup


def package_env(file_name, strict=False):
    file_path = path.join(path.dirname(__file__), file_name)
    if path.exists(file_path) or strict:
        return open(file_path).read()
    else:
        return ''


def parse_requirements(filename):
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


if __name__ == '__main__':
    setup(
        name='UISoup',
        version='2.5.6',
        description='Cross Platform GUI Test Automation tool.',
        long_description=package_env('README.rst'),
        author='Max Beloborodko',
        author_email='f1ashhimself@gmail.com',
        packages=['uisoup'],
        include_package_data=True,
        install_requires=parse_requirements('requirements.txt'),
        zip_safe=False,
        entry_points={
            'console_scripts': [
                'ui-inspector = uisoup.ui_inspector:main'
            ]
        }
    )
